#Fourier Series solution to heat equation with temp and flux boundary conditiond
#
#u_t   =  nu  u_xx                     0<x<L, t> 0:
#u(x,0) = u0(x)                  0<x<L:
#u_x(0,t) = theta1(t)     u (L,t) = theta2(t)      t>0:

restart:
L := 3:
nu := 2:
u0 := x -> 0:
theta1 := t -> 1;  # flux at top    (x = 0):
theta2 := t -> 0; # state at bottom (x = L):
mu := n -> (2*n-1)*Pi/(2*L):
S := (x,t) -> theta1(t)*(x-L) + theta2(t):
Sp := (x,t) -> diff(S(x,t),t):
Smode := (t,n) -> 2/L*int(S(x,t)*cos(mu(n)*x),x = 0..L):
Spmode := (t,n) -> 2/L*int(Sp(x,t)*cos(mu(n)*x),x = 0..L):
w0 := n -> 2/L*int((u0(x)-S(x,0))*cos(mu(n)*x),x = 0..L):
solh := (x,t,N) -> sum(cos(mu(n)*x)*w0(n)*exp(-(nu*mu(n)^2)*t),n=1..N):

solp := (x,t,N) -> sum( cos(mu(n)*x) * int(exp(-(nu*mu(n)^2)*(t-tau))*Spmode(tau,n),tau = 0 ..t),n=1..N):
u := (x,t,N) -> solh(x,t,N) + solp(x,t,N) + S(x,t):

ux := (x,t,N) -> diff(u(x,t,N),x):

# Flux at x = L:
g := (t,N) -> nu*(subs(x=L,ux(x,t,N)) - 1):
# state at x = 0:
h := (t,N) -> subs(x=0,u(x,t,N)):

ut := (x,t,N) -> diff(u(x,t,N),t):

# Now make  phi dual solution by  t ->  T- t:
# classical solution (same as forward, BCs compatible):

phi := (x,t,N,i) -> u(x,2^(-i)-t,N,i):
phix := (x,t,N,i) ->  diff(phi(x,t,N,i),x):

# Now solve psi dual problem:

# Functions for psi dual problem (classical solution):
psi0 := x -> 0:
t1 := t -> 0:  # flux at top    (x = 0):
t2 := t -> -t: # state at bottom (x = L):

# Shift function
S1 := (x,t) -> t2(t):
S1p := (x,t) -> diff(S1(x,t),t):
S1mode := (t,n) -> t2(t)*2/L*int(cos(mu(n)*x),x = 0..L):
S1pmode := (t,n) -> diff(t2(t),t)*2/L*int(cos(mu(n)*x),x = 0..L): 
solp1 := (x,t,N) -> sum( cos(mu(n)*x) * int(exp(-(nu*mu(n)^2)*(t-tau)) * S1pmode(tau,n),tau = 0 ..t),n=1..N) :
psi := (x,t,N) -> solp1(x,t,N) + S1(x,t):
psix := (x,t,N) -> diff(psi(x,t,N),x):

#Now compute rates for enties in matrix
m11 := (i,N) -> (Int(Int((ux(x,t,N)-1)*phix(x,t,N,i),x=0..L),t=0..2^(-i))):

m21 := (i,N) -> (Int(Int((ux(x,t,N)-1)*psix(x,2^(-i)-t,N),t = 0..2^(-i)),x=0..L)):

m12 := (i,N) -> (Int(Int(ut(x,t,N)*phi(x,t,N,i),x=0..L),t=0..2^(-i))):

m22 := (i,N) -> (Int(Int(ut(x,t,N)*psi(x,2^(-i)-t,N),x=0..L),t=0..2^(-i))):

b1 := (i,N) -> Int(g(t,N)*(t2(2^(-i)-t)),t=0..2^(-i)):
b2 := (i,N) -> Int(h(t,N)*(theta1(2^(-i)-t)),t=0..2^(-i)):

N := 10:
M := 5;

for i from 1 to M do:
 m1[i] := evalf(m11(i,N)):
 m2[i] := evalf(m12(i,N)):
 m3[i] := evalf(m21(i,N)):
 m4[i] := evalf(m22(i,N)):
 B1[i] := evalf(b1(i,N)):
 B2[i] := evalf(b2(i,N)):
 detM[i] := m1[i]*m4[i]-m2[i]*m3[i]:
 TOP1[i] := B1[i]*m4[i] - B2[i]*m2[i]:
 TOP2[i]:= m1[i]*B2[i] - m3[i]*B1[i]:
 DK[i] := TOP1[i]/detM[i]:
 DC[i] := TOP2[i]/detM[i]:
 print(m3):
 save m1,m2,m3,m4,B1,B2,detM,TOP1,TOP2,DK,DC, "jimmy_maple.m":
od:

m1list := convert([seq(eval(m1[i]),i=1..M)],string):
m2list := convert([seq(eval(m2[i]),i=1..M)],string):
m3list := convert([seq(eval(m3[i]),i=1..M)],string):
m4list := convert([seq(eval(m4[i]),i=1..M)],string):

b1list := convert([seq(eval(B1[i]),i=1..M)],string):
b2list := convert([seq(eval(B2[i]),i=1..M)],string):

detMlist := convert([seq(eval(detM[i]),i=1..M)],string):

T1list := convert([seq(eval(TOP1[i]),i=1..M)],string):
T2list := convert([seq(eval(TOP2[i]),i=1..M)],string):

dClist := convert([seq(eval(DC[i]),i=1..M)],string):
dKlist := convert([seq(eval(DK[i]),i=1..M)],string):

fid:= fopen("jimmy_data_maple.m",WRITE):
fprintf(fid,"%% Generated by MAPLE \n"):
fprintf(fid,"%% computes sequence of integrals \n"):
fprintf(fid,"%% for two parameter estimation \n"):
fprintf(fid,"%% iint = int_0^L int_0^t \n"):
fprintf(fid,"%% int = int_0^t \n"):
fprintf(fid,"%% m11 = iint (h_x-1) phi_x \n"):
fprintf(fid,"%% m21 = iint (h_x-1) psi_x \n"):
fprintf(fid,"%% m12 = iint h_t phi \n"):
fprintf(fid,"%% m22 = iint h_t psi \n"):
fprintf(fid,"%% b1 = int g t2(t-tau) dtau \n"):
fprintf(fid,"%% b2 = int h theta1(t-tau) dtau \n"):
fprintf(fid,"%% with t = 2^(-i) with i the index \n"):

fprintf(fid," \n"):
fprintf(fid," \n"):

fprintf(fid," m11 = %s ; \n",m1list):
fprintf(fid," m12 = %s ; \n",m2list):
fprintf(fid," m21 = %s ; \n",m3list):
fprintf(fid," m22 = %s ; \n",m4list):

fprintf(fid," \n"):

fprintf(fid," b1 = %s ; \n",b1list):
fprintf(fid," b2 = %s ; \n",b2list):

fprintf(fid," \n"):

fprintf(fid," top1 = %s ; \n",T1list):
fprintf(fid," top2 = %s ; \n",T2list):

fprintf(fid," \n"):

fprintf(fid," detM = %s ; \n",detMlist):

fprintf(fid," \n"):

fprintf(fid," dK = %s ; \n",dKlist):
fprintf(fid," dC = %s ; \n",dClist):

fclose(fid):
